declare global {
  namespace NodeJS {
    interface Process {
      type: string
    }
  }
}

const isNodeJS =
  typeof process === "object" &&
  process + "" === "[object process]" &&
  !process.versions.nw &&
  !(process.versions.electron && process.type && process.type !== "browser")

class BaseException extends Error{
  constructor(message: string, name: string = "BaseException") {
    super()
    if (this.constructor === BaseException) {
      throw new Error("Cannot construct BaseException instances directly")
    }
    this.message = message
    this.name = name
  }
}

class AbortException extends BaseException {
  constructor(message: string) {
    super(message, 'AbortException')
  }
}

class MissingPDFException extends BaseException {
  constructor(message: string) {
    super(message, 'MissingPDFException')
  }
}

class PasswordException extends BaseException {
  code: number

  constructor(message: string, code: number) {
    super(message, 'PasswordException')

    this.code = code
  }
}

class UnexpectedResponseException extends BaseException {
  status: number

  constructor(message: string, status: number) {
    super(message, 'UnexpectedResponseException')

    this.status = status
  }
}

class UnknownErrorException extends BaseException {
  details: string

  constructor(message: string, details: string) {
    super(message, 'UnknownErrorException')
    this.details = details
  }
}

export {
  isNodeJS,
  AbortException,
  MissingPDFException,
  PasswordException,
  UnexpectedResponseException,
  UnknownErrorException,
}
