import MessageHandler from "../shared/MessageHandler"
import { isNodeJS } from "../shared/util"

class WorkerMessageHandler {
  static setup(handler: MessageHandler, port: MessagePort) {
    let testMessageProcessed = false;
    handler.on("test", function (data: ArrayBufferLike) {
      if (testMessageProcessed) {
        return; // we already processed 'test' message once
      }
      testMessageProcessed = true;

      // Ensure that `TypedArray`s can be sent to the worker.
      handler.send("test", data instanceof Uint8Array);
    })

    handler.on("GetDocRequest", function (data) {
      return WorkerMessageHandler.createDocumentHandler(data, port);
    });
  }

  static initializeFromPort(port: MessagePort) {
    const handler = new MessageHandler("worker", "main", port);
    WorkerMessageHandler.setup(handler, port);
    handler.send("ready", null);
  }
}

function isMessagePort(maybePort: MessagePort): boolean {
  return(typeof maybePort.postMessage === 'function' && 'onmessage' in maybePort)
}

// Worker thread (and not Node.js)
// @ts-ignore
if (typeof window === 'undefined' &&
  !isNodeJS &&
  typeof self !== 'undefined' &&
  isMessagePort(self as unknown as MessagePort)) {
    WorkerMessageHandler.initializeFromPort(self as unknown as MessagePort)
}

export {
  WorkerMessageHandler
}
